package be.iminds.ilabt.jfed.testing.base;

import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticUserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.connection_pool.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.lowlevel.authority.legacy.TargetAuthority;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

/**
 * BaseApiTest adds some commonly used members to ApiTest:
 *  - JFedConnectionProvider;
 *  - TestbedInfoSource;
 *  - AuthorityFinder;
 *  - JFedPreferences;
 */
public abstract class BaseApiTest<SpecificApiTestConfig extends ApiTestConfig> extends ApiTest<SpecificApiTestConfig> {
    private final static org.slf4j.Logger LOG = LoggerFactory.getLogger(BaseApiTest.class);

    @Nonnull protected final JFedConnectionProvider connectionProvider;
    @Nonnull protected final TestbedInfoSource testbedInfoSource;
    @Nonnull protected final AuthorityFinder authorityFinder;
    @Nonnull private final JFedPreferences jFedPreferences;


    //all overriding classes should use @Inject for their constructors!
    public BaseApiTest(@Nonnull Logger logger,
                       @Nullable TargetAuthority testedAuthority,
                       @Nullable GeniUserProvider geniUserProvider,
                       @Nonnull SpecificApiTestConfig testConfig,
                       @Nonnull JFedConnectionProvider connectionProvider,
                       @Nonnull TestbedInfoSource testbedInfoSource,
                       @Nonnull AuthorityFinder authorityFinder,
                       @Nonnull JFedPreferences jFedPreferences) {
        super(logger, testedAuthority, geniUserProvider, testConfig);
        this.connectionProvider = connectionProvider;
        this.testbedInfoSource = testbedInfoSource;
        this.authorityFinder = authorityFinder;
        this.jFedPreferences = jFedPreferences;
    }

    //all overriding classes should use @Inject for their constructors!
    public BaseApiTest(@Nonnull Logger logger,
                       @Nullable TargetAuthority testedAuthority,
                       @Nullable GeniUserProvider geniUserProvider,
                       @Nonnull SpecificApiTestConfig testConfig,
                       @Nonnull JFedConnectionProvider connectionProvider,
                       @Nonnull TestbedInfoSource testbedInfoSource,
                       @Nonnull JFedPreferences jFedPreferences) {
        this(logger, testedAuthority, geniUserProvider, testConfig,
                connectionProvider, testbedInfoSource,
                new AuthorityFinder(testbedInfoSource, jFedPreferences) /*safe fallback as AuthorityFinder does not keep state*/,
                jFedPreferences);
    }


    @Nonnull public JFedConnectionProvider getConnectionProvider() {
        return connectionProvider;
    }

    @Nonnull public TestbedInfoSource getTestbedInfoSource() {
        return testbedInfoSource;
    }

    public JFedPreferences getJFedPreferences() {
        return jFedPreferences;
    }


    public AutomaticUserAndSliceApiWrapper.AutomaticUserAndSliceApiWrapperFactory getAutomaticUserAndSliceApiWrapperFactory() {
        return new AutomaticUserAndSliceApiWrapper.BasicAutomaticUserAndSliceApiWrapperFactory(logger, geniUserProvider, connectionProvider, getJFedPreferences());
    }

    public AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory getAutomaticAggregateManagerWrapperFactory() {
        return new AutomaticAggregateManagerWrapper.BasicAutomaticAggregateManagerWrapperFactory(logger, geniUserProvider, connectionProvider, getJFedPreferences());
    }
}

